% Diplomarbeit Johannes Loose
% Octave Version 3.0.0
% Dieses Programm ist gemeinfrei.
% Author, Umsetzung des Rechenwegs in Octave:
% 2008-04-20 - Philipp Edelmann, Nagoya, Japan
% Methodik, Pruefung, Erweiterung und Statistik:
% 2008-04-21 - Johannes Loose, Freising, Germany

function statistik(k, anzfl, outputpath)
% Statistische Auswertung verschiedener
% Fehlerschaetzverfahren bei polaren Aufnahmen
%
% k ist die Anzahl der Wiederholungen pro Flaeche
% anzfl ist die Anzahl der Flaechen
% outputpath ist der Pfad und Dateiname in die geschrieben werden soll
% Ist der Wert 0, so erfolgt die Ausgabe nur auf dem Bildschirm
%
% Beispiel: statistik(100,100,"output/output.txt") - Ausgabe in Datei  
% oder: statistik(100,100,0) - Ausgabe auf dem Bildschirm

% Standardabweichungen fuer Laenge und Winkel
% relative Streckenabweichung
sr=0.045;
% absolute Winkelabweichung
sp=1/180*pi;

% Schaetzungen fuer die Standardabweichungen
sr_est=0.045;
sp_est=1/180*pi;

% Schranke fuer Laengenfehler/Umfang
schranke1=0.02;
% Schranke fuer Flaechenfehler/Gesamtflaeche = 'Verbesserung'
schranke2=0.07;
% Schranke fuer den neuen Schaetzer
schranke3=0.1;
% Schranke fuer den Gaussschen Schaetzer
schranke4=0.05;
% Schranke fuer den wahren Fehler
schrankew=0.05;

% Laden der Sollflaechen aus Dateien
pol=laden(anzfl);

for i=1:length(pol)
	
	% Sollflaeche
	cart{i}=tocart(pol{i});
	F(i)=flaeche(cart{i});
	
	
	for j=1:k
		% Simulation des Messfehlers
		polf{i,j}=prod_fehler(pol{i},sr,sp);
		cartf{i,j}=tocart(polf{i,j});
		
		% Abschlussfehler, absoluter Laengenfehler (Ursprung immer bei 0/0)
		anschl(i,j)=norm(cartf{i,j}(length(cartf{i,j}),:));
		
		% FPGO / Werte vor streckenproportionaler Fehlerverteilung
		Ff(i,j)=flaeche(cartf{i,j});
		
		% FPGG / Werte nach streckenproportionaler Fehlerverteilung
		[cartfv{i,j},rcartfv]=fehlerverteilung(polf{i});
		Ffv(i,j)=flaeche(cartfv{i,j});

		% Umfang
		umf(i,j)=umfang(rcartfv);
		
		% neuer Schaetzer
		abwn(i,j)=neuer_schaetzer(polf{i,j},sr,sp);

		% Fehler nach Gaussscher Fehlerfortpflanzung mit angenommenen Messungenauigkeiten
		fehlergauss(i,j)=Ffehler(polf{i,j},sr_est,sp_est);
	end
end

% Kriterien zur Pruefung
% Abschlussfehler / Umfang = 'relativer Laengenfehler'
krit1=anschl./umf<schranke1;
% |FPGO - FPGG| = 'Verbesserung'
krit2=abs(Ff-Ffv)./Ffv<schranke2;
% 'Schaetzer'
krit3=(abwn./Ffv)<schranke3;
% 'Gausssche Fehlerfortpflanzung'
krit4=fehlergauss./Ffv<schranke4;
% 'realer Fehler'
kritw=abs(F'*ones(1,k)-Ffv)./(F'*ones(1,k))<schrankew;

% Summe erkannt, nicht erkannt, Fehlalarm
ges1=mean(mean((!krit1&!kritw)'))+mean(mean((krit1&!kritw)'))+mean(mean((!krit1&kritw)'));
ges2=mean(mean((!krit2&!kritw)'))+mean(mean((krit2&!kritw)'))+mean(mean((!krit2&kritw)'));
ges3=mean(mean((!krit3&!kritw)'))+mean(mean((krit3&!kritw)'))+mean(mean((!krit3&kritw)'));
ges4=mean(mean((!krit4&!kritw)'))+mean(mean((krit4&!kritw)'))+mean(mean((!krit4&kritw)'));


% Ausgabe auf dem Bildschirm
printf("------------------------------------------------------------------------\n")
printf("Diplomarbeit Johannes Loose\n")
printf("Statistische Auswertung verschiedener\n")
printf("Fehlerschaetzverfahren bei polaren Aufnahmen\n")
printf("------------------------------------------------------------------------\n\n")

printf("Anzahl der Flaechen: %d. Anzahl Messfehler pro Flaeche: %d.\n\n",k,anzfl)

printf("---------------------------Eingangsvariablen----------------------------\n")
printf("Standardabweichungen fuer Laenge und Winkel\n")
printf("relative Streckenabweichung:	%8.2f%%\n",sr*100)
printf("absolute Winkelabweichung:	%8.2f Grad\n\n",sp*180/pi)
printf("Standardabweichungen der angenommenen Messunsicherheiten fuer die\nGaussche Fehlerfortpflanzung\n")
printf("relative Streckenabweichung:	%8.2f%%\n",sr_est*100)
printf("absolute Winkelabweichung:	%8.2f Grad\n\n",sp_est*180/pi)

printf("Akzeptierte Obergrenzen der Fehlerwerte\n")
printf("relativer Laengenfehler		%8.2f%%\n",schranke1*100)
printf("Verbesserung			%8.2f%%\n",schranke2*100)
printf("Schaetzer			%8.2f%%\n",schranke3*100)
printf("Gaussche F.			%8.2f%%\n",schranke4*100)
printf("realer Fehler			%8.2f%%\n",schrankew*100)
printf("------------------------------------------------------------------------\n\n")
printf("---------------------------Berechnete Werte-----------------------------\n")
printf("			Mittelwert	Standard-	Mittelwert\n					abweichung	Fehler<Grenze\n")
printf("relativer Laengenfehler %f	%f	%8.0f%%\n",mean(mean(anschl./umf)),mean(std(anschl./umf)),mean(mean(krit1)*100))
printf("Verbesserung		%f	%f	%8.0f%%\n",mean(mean((Ff-Ffv)./Ffv)), mean(std((Ff-Ffv)./Ffv)), mean(mean(krit2)*100))
printf("Schaetzer		%f	%f	%8.0f%%\n",mean(mean(fehlergauss./Ffv)),mean(std(fehlergauss./Ffv)), mean(mean(krit3)*100))
printf("Gaussche F.		%f	%f	%8.0f%%\n",mean(mean(abwn./Ffv)), mean(std(abwn./Ffv)), mean(mean(krit4)*100))
printf("realer Fehler		%f	%f	%8.0f%%\n\n",mean(mean(abs(F'*ones(1,k)-Ffv)./(F'*ones(1,k)))), mean(std(abs(F'*ones(1,k)-Ffv)./(F'*ones(1,k)))), mean((mean(kritw)*100)))

printf("Aufgetretene Faelle\n")
printf("		Fehler	Fehler	Fehl-	kein	Aussage\n")
printf("		erkannt nicht	alarm	Fehler	korrekt\n")
printf("			erkannt\n")
%printf("x		!x&!w	x&!w	!x&w	!x&!w	x==w\n")
printf("Laengenfehler	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit1&!kritw)')),ges1)*100,defzero(mean(mean((krit1&!kritw)')),ges1)*100,defzero(mean(mean((!krit1&kritw)')),ges1)*100,mean(mean((krit1&kritw)'))*100,mean(mean((krit1==kritw)'))*100)
printf("Verbesserung	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit2&!kritw)')),ges2)*100,defzero(mean(mean((krit2&!kritw)')),ges2)*100,defzero(mean(mean((!krit2&kritw)')),ges2)*100,mean(mean((krit2&kritw)'))*100,mean(mean((krit2==kritw)'))*100)
printf("Schaetzer	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit3&!kritw)')),ges3)*100,defzero(mean(mean((krit3&!kritw)')),ges3)*100,defzero(mean(mean((!krit3&kritw)')),ges3)*100,mean(mean((krit3&kritw)'))*100,mean(mean((krit3==kritw)'))*100)
printf("Gaussche F.	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit4&!kritw)')),ges4)*100,defzero(mean(mean((krit4&!kritw)')),ges4)*100,defzero(mean(mean((!krit4&kritw)')),ges4)*100,mean(mean((krit4&kritw)'))*100,mean(mean((krit4==kritw)'))*100)
printf("------------------------------------------------------------------------\n")

% Ausgabe in Datei, wenn Dateiname angegeben

if (outputpath)
	myfile = fopen(outputpath, 'wt');
	fprintf(myfile,"------------------------------------------------------------------------\n")
	fprintf(myfile,"Diplomarbeit Johannes Loose\n")
	fprintf(myfile,"Statistische Auswertung verschiedener\n")
	fprintf(myfile,"Fehlerschaetzverfahren bei polaren Aufnahmen\n")
	fprintf(myfile,"------------------------------------------------------------------------\n\n")
	
	fprintf(myfile,"Anzahl der Flaechen: %d. Anzahl Messfehler pro Flaeche: %d.\n\n",k,anzfl)

	fprintf(myfile,"---------------------------Eingangsvariablen----------------------------\n")
	fprintf(myfile,"Standardabweichungen fuer Laenge und Winkel\n")
	fprintf(myfile,"relative Streckenabweichung:	%8.2f%%\n",sr*100)
	fprintf(myfile,"absolute Winkelabweichung:	%8.2f Grad\n\n",sp*180/pi)
	fprintf(myfile,"Standardabweichungen der angenommenen Messunsicherheiten fuer die\nGaussche Fehlerfortpflanzung\n")
	fprintf(myfile,"relative Streckenabweichung:	%8.2f%%\n",sr_est*100)
	fprintf(myfile,"absolute Winkelabweichung:	%8.2f Grad\n\n",sp_est*180/pi)

	fprintf(myfile,"Akzeptierte Obergrenzen der Fehlerwerte\n")
	fprintf(myfile,"relativer Laengenfehler		%8.2f%%\n",schranke1*100)
	fprintf(myfile,"Verbesserung			%8.2f%%\n",schranke2*100)
	fprintf(myfile,"Schaetzer			%8.2f%%\n",schranke3*100)
	fprintf(myfile,"Gaussche F.			%8.2f%%\n",schranke4*100)
	fprintf(myfile,"realer Fehler			%8.2f%%\n",schrankew*100)
	fprintf(myfile,"------------------------------------------------------------------------\n\n")
	fprintf(myfile,"---------------------------Berechnete Werte-----------------------------\n")
	fprintf(myfile,"			Mittelwert	Standard-	Mittelwert\n					abweichung	Fehler<Grenze\n")
	fprintf(myfile,"relativer Laengenfehler %f	%f	%8.0f%%\n",mean(mean(anschl./umf)),mean(std(anschl./umf)),mean(mean(krit1)*100))
	fprintf(myfile,"Verbesserung		%f	%f	%8.0f%%\n",mean(mean((Ff-Ffv)./Ffv)), mean(std((Ff-Ffv)./Ffv)), mean(mean(krit2)*100))
	fprintf(myfile,"Schaetzer		%f	%f	%8.0f%%\n",mean(mean(fehlergauss./Ffv)),mean(std(fehlergauss./Ffv)), mean(mean(krit3)*100))
	fprintf(myfile,"Gaussche F.		%f	%f	%8.0f%%\n",mean(mean(abwn./Ffv)), mean(std(abwn./Ffv)), mean(mean(krit4)*100))
	fprintf(myfile,"realer Fehler		%f	%f	%8.0f%%\n\n",mean(mean(abs(F'*ones(1,k)-Ffv)./(F'*ones(1,k)))), mean(std(abs(F'*ones(1,k)-Ffv)./(F'*ones(1,k)))), mean((mean(kritw)*100)))

	fprintf(myfile,"Aufgetretene Faelle\n")
	fprintf(myfile,"		Fehler	Fehler	Fehl-	kein	Aussage\n")
	fprintf(myfile,"		erkannt nicht	alarm	Fehler	korrekt\n")
	fprintf(myfile,"			erkannt\n")
	%fprintf(myfile,"x		!x&!w	x&!w	!x&w	!x&!w	x==w\n")
	fprintf(myfile,"Laengenfehler	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit1&!kritw)')),ges1)*100,defzero(mean(mean((krit1&!kritw)')),ges1)*100,defzero(mean(mean((!krit1&kritw)')),ges1)*100,mean(mean((krit1&kritw)'))*100,mean(mean((krit1==kritw)'))*100)
	fprintf(myfile,"Verbesserung	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit2&!kritw)')),ges2)*100,defzero(mean(mean((krit2&!kritw)')),ges2)*100,defzero(mean(mean((!krit2&kritw)')),ges2)*100,mean(mean((krit2&kritw)'))*100,mean(mean((krit2==kritw)'))*100)
	fprintf(myfile,"Schaetzer	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit3&!kritw)')),ges3)*100,defzero(mean(mean((krit3&!kritw)')),ges3)*100,defzero(mean(mean((!krit3&kritw)')),ges3)*100,mean(mean((krit3&kritw)'))*100,mean(mean((krit3==kritw)'))*100)
	fprintf(myfile,"Gaussche F.	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%	%3.0f%%\n",defzero(mean(mean((!krit4&!kritw)')),ges4)*100,defzero(mean(mean((krit4&!kritw)')),ges4)*100,defzero(mean(mean((!krit4&kritw)')),ges4)*100,mean(mean((krit4&kritw)'))*100,mean(mean((krit4==kritw)'))*100)
	fprintf(myfile,"------------------------------------------------------------------------\n")
	fclose(myfile);
endif